import { HttpClientBackendService, getStatusText, STATUS, removeTrailingSlash, InMemoryDbService, InMemoryBackendConfig } from 'angular-in-memory-web-api';
import { HttpResponse, HttpHeaders, HttpErrorResponse, HttpParams, HttpBackend, XhrFactory } from '@angular/common/http';
import { delayResponse } from 'angular-in-memory-web-api/delay-response';
import { of, Observable } from 'rxjs';
import { v4 } from 'uuid';
import { NgModule } from '@angular/core';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @param {?} req
 * @return {?}
 */
function getRequestMethod(req) {
    if (req.url.endsWith('/alterar') || req.url.endsWith('/editar')) {
        return 'put';
    }
    else if (req.url.endsWith('/excluir') || req.url.endsWith('/deletar')) {
        return 'delete';
    }
    else {
        return (req.method || 'get').toLowerCase();
    }
}
// tslint:disable: no-string-literal
/**
 * @param {?} dbService
 * @param {?} options
 * @param {?} xhrFactory
 * @return {?}
 */
function httpClientInMemBackendServiceFactory(dbService, options, xhrFactory) {
    /** @type {?} */
    const backend = new HttpClientBackendService(dbService, options, xhrFactory);
    backend['getRequestMethod'] = getRequestMethod.bind(backend);
    return backend;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// tslint:disable: no-string-literal
class InMemoryDataService {
    constructor() {
        this.loadsFn = [];
        this.requestInterceptors = [];
        this.replaceMap = new Map();
        this.searchTermMap = new Map();
        this.transformGetMap = new Map();
        this.transformPostMap = new Map();
        this.transformPutMap = new Map();
        this.fieldsFilterMap = new Map();
        this.dbe2e = new Map();
        this.responseUtils = this.getResponseUtils();
        this.config = getMemoryConfig();
        console.log('[In-Memory]', 'Configuration: ', this.config);
        window['inMemoryDataService'] = this;
    }
    /**
     * @return {?}
     */
    createDb() {
        return of({});
    }
    /**
     * @return {?}
     */
    deleteDatabase() {
        /** @type {?} */
        const result$ = new Promise((/**
         * @param {?} resolve
         * @param {?} reject
         * @return {?}
         */
        (resolve, reject) => {
            // Para testes e2e usa o banco de dados em memória mesmo
            if (this.config.memoryOnly) {
                console.log('[In-Memory]', 'Deleted e2e OK!');
                resolve(true);
            }
            else {
                /** @type {?} */
                const request = indexedDB.deleteDatabase(this.config.databaseName);
                /** @type {?} */
                const errorResponse = (/**
                 * @param {?} event
                 * @return {?}
                 */
                (event) => {
                    console.log('[In-Memory]', 'Error deleting the DB');
                    /** @type {?} */
                    const mensagem = 'Não foi possível excluir o banco de dados da memória.\n' +
                        'Verifique se você está com outra aba aberta com a aplicação.\n' +
                        'Este comportamento é necessário para ter a garantia de sempre recriar o banco de dados.';
                    alert(mensagem);
                    reject(((/** @type {?} */ (event.target))).errorCode);
                });
                request.onblocked = (/**
                 * @param {?} event
                 * @return {?}
                 */
                (event) => {
                    errorResponse(event);
                });
                request.onerror = (/**
                 * @param {?} event
                 * @return {?}
                 */
                (event) => {
                    errorResponse(event);
                });
                request.onsuccess = (/**
                 * @return {?}
                 */
                () => {
                    console.log('[In-Memory]', 'Deleted OK.');
                    // alert('*** NOTE : Requires page refresh to see the DB removed from the Resources IndexedDB tab in Chrome.');
                    resolve(true);
                });
            }
        }));
        return result$;
    }
    /**
     * @return {?}
     */
    createDatabase() {
        /** @type {?} */
        const self = this;
        /** @type {?} */
        const result$ = new Promise((/**
         * @param {?} resolve
         * @param {?} reject
         * @return {?}
         */
        (resolve, reject) => {
            // Para testes e2e usa o banco de dados em memória mesmo
            if (this.config.memoryOnly) {
                console.log('[In-Memory]', 'Database e2e created!');
                resolve(true);
            }
            else {
                /** @type {?} */
                const request = window.indexedDB.open(this.config.databaseName, 1);
                request.onsuccess = (/**
                 * @param {?} event
                 * @return {?}
                 */
                (event) => {
                    self.db = request.result;
                    console.log('[In-Memory]', 'Database created!');
                });
                request.onerror = (/**
                 * @param {?} event
                 * @return {?}
                 */
                (event) => {
                    console.log('[In-Memory]', 'Database error: ', ((/** @type {?} */ (event.target))).errorCode);
                    reject(((/** @type {?} */ (event.target))).errorCode);
                });
                request.onupgradeneeded = (/**
                 * @param {?} event
                 * @return {?}
                 */
                (event) => {
                    self.db = request.result;
                    console.log('[In-Memory]', 'Database upgrade nedded!');
                    resolve(true);
                });
            }
        }));
        return result$;
    }
    /**
     * @param {?} dataServiceFn
     * @return {?}
     */
    createObjectStore(dataServiceFn) {
        /** @type {?} */
        const self = this;
        /** @type {?} */
        const result$ = new Promise((/**
         * @param {?} resolve
         * @param {?} reject
         * @return {?}
         */
        (resolve, reject) => {
            // Para testes e2e usa o banco de dados em memória mesmo
            if (this.config.memoryOnly) {
                dataServiceFn.forEach((/**
                 * @param {?} loadFn
                 * @param {?} name
                 * @return {?}
                 */
                (loadFn, name) => {
                    console.log('[In-Memory]', `Added: ${name}.`);
                    this.dbe2e.set(name, []);
                    if (loadFn && loadFn instanceof Function) {
                        loadFn.call(null, self);
                    }
                }));
                resolve(true);
            }
            else {
                /** @type {?} */
                let objectStore;
                dataServiceFn.forEach((/**
                 * @param {?} loadFn
                 * @param {?} name
                 * @return {?}
                 */
                (loadFn, name) => {
                    console.log('[In-Memory]', `${name} => Mock defined`);
                    objectStore = this.db.createObjectStore(name, { keyPath: 'id' });
                    if (loadFn && loadFn instanceof Function) {
                        self.loadsFn.push(loadFn);
                    }
                }));
                objectStore.transaction.oncomplete = (/**
                 * @param {?} event
                 * @return {?}
                 */
                (event) => {
                    self.loadsFn.forEach((/**
                     * @param {?} fn
                     * @return {?}
                     */
                    fn => {
                        fn.call(null, self);
                    }));
                    resolve(true);
                });
                objectStore.transaction.onerror = (/**
                 * @param {?} event
                 * @return {?}
                 */
                (event) => {
                    reject(event);
                });
            }
        }));
        return result$;
    }
    /**
     * @param {?} collectionName
     * @param {?} data
     * @return {?}
     */
    storeData(collectionName, data) {
        /** @type {?} */
        const self = this;
        /** @type {?} */
        const result$ = new Promise((/**
         * @param {?} resolve
         * @param {?} reject
         * @return {?}
         */
        (resolve, reject) => {
            if (!data['id']) {
                data['id'] = v4();
            }
            // Para testes e2e usa o banco de dados em memória mesmo
            if (this.config.memoryOnly) {
                /** @type {?} */
                const objectStore = self.dbe2e.get(collectionName);
                objectStore.push(data);
                resolve(data['id']);
            }
            else {
                /** @type {?} */
                const objectStore = self.db.transaction(collectionName, 'readwrite').objectStore(collectionName);
                /** @type {?} */
                const request = objectStore.add(data);
                request.onsuccess = (/**
                 * @param {?} event
                 * @return {?}
                 */
                (event) => {
                    resolve((request.result !== undefined) ? data['id'] : null);
                });
                request.onerror = (/**
                 * @param {?} event
                 * @return {?}
                 */
                (event) => {
                    reject(event);
                });
            }
        }));
        return result$;
    }
    /**
     * @param {?} collectionName
     * @return {?}
     */
    clearData(collectionName) {
        /** @type {?} */
        const self = this;
        /** @type {?} */
        const result$ = new Promise((/**
         * @param {?} resolve
         * @param {?} reject
         * @return {?}
         */
        (resolve, reject) => {
            // Para testes e2e usa o banco de dados em memória mesmo
            if (this.config.memoryOnly) {
                self.dbe2e.set(collectionName, []);
                resolve(true);
            }
            else {
                /** @type {?} */
                const objectStore = self.db.transaction(collectionName, 'readwrite').objectStore(collectionName);
                /** @type {?} */
                const request = objectStore.clear();
                request.onsuccess = (/**
                 * @param {?} event
                 * @return {?}
                 */
                (event) => {
                    resolve(true);
                });
                request.onerror = (/**
                 * @param {?} event
                 * @return {?}
                 */
                (event) => {
                    reject(event);
                });
            }
        }));
        return result$;
    }
    /**
     * @param {?} requestInterceptor
     * @return {?}
     */
    addRequestInterceptor(requestInterceptor) {
        if (!requestInterceptor.method) {
            requestInterceptor['method'] = 'GET';
        }
        this.requestInterceptors.push(requestInterceptor);
    }
    /**
     * @param {?} key
     * @param {?} value
     * @return {?}
     */
    addReplaceUrl(key, value) {
        this.replaceMap.set(key, value);
    }
    /**
     * @param {?} collectionName
     * @param {?} fields
     * @return {?}
     */
    addSearchTermMap(collectionName, fields) {
        this.searchTermMap.set(collectionName, fields);
    }
    /**
     * @param {?} collectionName
     * @param {?} transformfn
     * @return {?}
     */
    addTransformGetMap(collectionName, transformfn) {
        this.transformGetMap.set(collectionName, transformfn);
    }
    /**
     * @param {?} collectionName
     * @param {?} transformfn
     * @return {?}
     */
    addTransformPostMap(collectionName, transformfn) {
        this.transformPostMap.set(collectionName, transformfn);
    }
    /**
     * @param {?} collectionName
     * @param {?} transformfn
     * @return {?}
     */
    addTransformPutMap(collectionName, transformfn) {
        this.transformPutMap.set(collectionName, transformfn);
    }
    /**
     * @param {?} collectionName
     * @param {?} field
     * @param {?} filterfn
     * @return {?}
     */
    addFieldFilterMap(collectionName, field, filterfn) {
        /** @type {?} */
        let fieldsFilterMap = this.fieldsFilterMap.get(collectionName);
        if (fieldsFilterMap !== undefined) {
            fieldsFilterMap.set(field, filterfn);
        }
        else {
            fieldsFilterMap = new Map();
            fieldsFilterMap.set(field, filterfn);
            this.fieldsFilterMap.set(collectionName, fieldsFilterMap);
        }
    }
    /**
     * @param {?} value
     * @return {?}
     */
    addRequestInterceptorByValue(value) {
        /** @type {?} */
        let obj;
        /** @type {?} */
        let response;
        if (typeof value === 'string') {
            try {
                obj = JSON.parse(value);
            }
            catch (error) {
                console.error('[In-Memory]', 'O valor informado não é possível de ser interpretado como uma interface IRequestInterceptor');
                return;
            }
        }
        else {
            obj = value;
        }
        if (obj && obj.path && typeof obj.path === 'string' && obj.response) {
            /** @type {?} */
            const path = '/' + obj.path.replace(/^\//, '');
            /** @type {?} */
            const url = window.location.origin + path;
            /** @type {?} */
            const status = (obj.response.status && typeof obj.response.status === 'number') ? obj.response.status :
                (obj.response.statusCode && typeof obj.response.statusCode === 'number') ? obj.response.statusCode :
                    (obj.response.error) ? 400 : 200;
            if (typeof obj.response === 'string') {
                response = new HttpResponse({
                    body: obj.response,
                    url,
                    headers: new HttpHeaders({ 'Content-Type': 'text/html; charset=utf-8' }),
                    status,
                    statusText: getStatusText(status)
                });
            }
            else if (obj.response.error) {
                response = new HttpErrorResponse({
                    error: obj.response.error,
                    url,
                    headers: new HttpHeaders({ 'Content-Type': 'application/json' }),
                    status,
                    statusText: getStatusText(status)
                });
            }
            else {
                response = new HttpResponse({
                    body: (obj.response.body) ? obj.response.body : obj.response,
                    url,
                    headers: new HttpHeaders({ 'Content-Type': 'application/json' }),
                    status,
                    statusText: getStatusText(status)
                });
            }
        }
        else {
            console.error('[In-Memory]', 'O valor informado não é possível de ser interpretado como uma interface IRequestInterceptor');
            return;
        }
        /** @type {?} */
        const requestInterceptor = {
            method: obj.method ? obj.method : 'GET',
            path: obj.path,
            response,
        };
        /** @type {?} */
        let params;
        /** @type {?} */
        const query = obj.query ? obj.query :
            (obj.queryStringParameters ? obj.queryStringParameters : undefined);
        if (query) {
            if (typeof query === 'string') {
                params = new HttpParams({ fromString: query });
            }
            else {
                params = new HttpParams({ fromObject: query });
            }
            if (params && params.keys.length > 0) {
                requestInterceptor['query'] = params;
            }
        }
        this.requestInterceptors.push(requestInterceptor);
    }
    /**
     * @private
     * @param {?} collectionName
     * @param {?} field
     * @return {?}
     */
    getFieldFilterMap(collectionName, field) {
        /** @type {?} */
        const fieldsFilterMap = this.fieldsFilterMap.get(collectionName);
        if (fieldsFilterMap !== undefined) {
            return fieldsFilterMap.get(field);
        }
        else {
            return undefined;
        }
    }
    /**
     * @private
     * @param {?} data
     * @return {?}
     */
    clone(data) {
        return JSON.parse(JSON.stringify(data));
    }
    /**
     * @private
     * @param {?} url
     * @param {?} status
     * @param {?} message
     * @param {?=} detailedMessage
     * @return {?}
     */
    createErrorResponseOptions(url, status, message, detailedMessage) {
        return new HttpErrorResponse({
            error: { message, detailedMessage },
            url,
            headers: new HttpHeaders({ 'Content-Type': 'application/json' }),
            status,
            statusText: getStatusText(status)
        });
    }
    /**
     * @private
     * @param {?} url
     * @param {?} status
     * @param {?=} body
     * @return {?}
     */
    createResponseOptions(url, status, body) {
        return new HttpResponse({
            body,
            url,
            headers: new HttpHeaders({ 'Content-Type': 'application/json' }),
            status,
            statusText: getStatusText(status)
        });
    }
    /**
     * @private
     * @param {?} item
     * @param {?} conditions
     * @param {?} useFilterOr
     * @return {?}
     */
    filterItem(item, conditions, useFilterOr) {
        if (conditions === undefined) {
            return true;
        }
        return useFilterOr ? this.filterItemOr(item, conditions) : this.filterItemAnd(item, conditions);
    }
    /**
     * @private
     * @param {?} item
     * @param {?} conditions
     * @return {?}
     */
    filterItemAnd(item, conditions) {
        /** @type {?} */
        let ok = true;
        /** @type {?} */
        let i = conditions.length;
        /** @type {?} */
        let cond;
        while (ok && i) {
            i -= 1;
            cond = conditions[i];
            if (cond.fn) {
                ok = cond.fn.call(this, item);
            }
            else {
                ok = cond.rx.test(item[cond.name]);
            }
        }
        return ok;
    }
    /**
     * @private
     * @param {?} item
     * @param {?} conditions
     * @return {?}
     */
    filterItemOr(item, conditions) {
        /** @type {?} */
        let ok = false;
        /** @type {?} */
        let okUnidade = true;
        /** @type {?} */
        let i = conditions.length;
        /** @type {?} */
        let cond;
        while (okUnidade && !ok && i) {
            i -= 1;
            cond = conditions[i];
            if (cond.name === 'unidadeId') {
                okUnidade = cond.fn.call(this, item);
            }
            else {
                if (cond.fn) {
                    ok = cond.fn.call(this, item);
                }
                else {
                    ok = cond.rx.test(item[cond.name]);
                }
            }
        }
        return ok && okUnidade;
    }
    /**
     * @private
     * @param {?} value
     * @param {?} filterFn
     * @return {?}
     */
    createFilterFn(value, filterFn) {
        return (/**
         * @param {?} item
         * @return {?}
         */
        (item) => {
            return filterFn.call(this, value, item);
        });
    }
    /**
     * @private
     * @param {?} id
     * @return {?}
     */
    createFilterUnidadeFn(id) {
        return (/**
         * @param {?} item
         * @return {?}
         */
        (item) => {
            return item['unidadeId'] === id;
        });
    }
    /**
     * @private
     * @param {?} field
     * @param {?} value
     * @return {?}
     */
    createFilterArrayFn(field, value) {
        return (/**
         * @param {?} item
         * @return {?}
         */
        (item) => {
            return value.includes(item[field]);
        });
    }
    /**
     * @private
     * @param {?} response$
     * @param {?} delay
     * @return {?}
     */
    addDelay(response$, delay) {
        return delay === 0 ? response$ : delayResponse(response$, (Math.floor((Math.random() * delay) + 1)) || 500);
    }
    /**
     * @private
     * @param {?} collectionName
     * @param {?} query
     * @param {?} caseSensitive
     * @return {?}
     */
    getQueryParams(collectionName, query, caseSensitive) {
        /** @type {?} */
        const queryParams = { count: 0 };
        query.forEach((/**
         * @param {?} value
         * @param {?} name
         * @return {?}
         */
        (value, name) => {
            if (name === 'page') {
                queryParams['page'] = parseInt(value[0], 10);
            }
            else if (name === 'pageSize') {
                queryParams['pageSize'] = parseInt(value[0], 10);
            }
            else if (name === 'unidadeId') {
                /** @type {?} */
                const condition = { name };
                if (queryParams['conditions'] === undefined) {
                    queryParams['conditions'] = [];
                }
                condition['fn'] = this.createFilterUnidadeFn(value[0]);
                queryParams.conditions.push(condition);
            }
            else if (name === 'searchTerm' && value[0]) {
                queryParams['useFilterOr'] = true;
                /** @type {?} */
                const fields = this.searchTermMap.get(collectionName);
                if (fields !== undefined) {
                    if (queryParams['conditions'] === undefined) {
                        queryParams['conditions'] = [];
                    }
                    queryParams.conditions.push(...fields.map((/**
                     * @param {?} field
                     * @return {?}
                     */
                    (field) => {
                        return { name: field, rx: new RegExp(decodeURI(value[0]), caseSensitive) };
                    })));
                }
            }
            else if (name !== 'order' && name !== 'fields' && name !== '$filter') {
                if (queryParams['conditions'] === undefined) {
                    queryParams['conditions'] = [];
                }
                /** @type {?} */
                const condition = { name };
                /** @type {?} */
                const filterFn = this.getFieldFilterMap(collectionName, name);
                if (filterFn !== undefined && filterFn instanceof Function) {
                    condition['fn'] = this.createFilterFn(value.length > 1 ? value : decodeURI(value[0]), filterFn);
                }
                else if (value.length > 1) {
                    condition['fn'] = this.createFilterArrayFn(name, value);
                }
                else {
                    condition['rx'] = new RegExp(decodeURI(value[0]), caseSensitive);
                }
                queryParams.conditions.push(condition);
            }
        }));
        return queryParams;
    }
    /**
     * @private
     * @param {?} cursor
     * @param {?} queryResults
     * @param {?} queryParams
     * @param {?} transformfn
     * @return {?}
     */
    getAllItems(cursor, queryResults, queryParams, transformfn) {
        /** @type {?} */
        let retorna = false;
        if (cursor) {
            /** @type {?} */
            let item = cursor.value;
            if (this.filterItem(item, queryParams.conditions, queryParams.useFilterOr)) {
                if (queryParams.page && queryParams.pageSize) {
                    if (queryParams.count < ((queryParams.page - 1) * queryParams.pageSize)) {
                        queryParams.count++;
                        cursor.continue();
                    }
                    else if (queryParams.count < (queryParams.page * queryParams.pageSize)) {
                        if (transformfn !== undefined) {
                            item = transformfn.call(this, item, this);
                        }
                        queryResults.items.push(item);
                        queryResults.hasNext = true;
                        queryParams.count++;
                        cursor.continue();
                    }
                    else {
                        retorna = true;
                    }
                }
                else {
                    if (transformfn !== undefined) {
                        item = transformfn.call(this, item, this);
                    }
                    queryResults.items.push(item);
                    queryResults.hasNext = true;
                    cursor.continue();
                }
            }
            else {
                cursor.continue();
            }
        }
        else {
            queryResults.hasNext = false;
            retorna = true;
        }
        return retorna;
    }
    /**
     * @private
     * @template T
     * @param {?} collection
     * @param {?} id
     * @return {?}
     */
    findById(collection, id) {
        return collection.find((/**
         * @param {?} item
         * @return {?}
         */
        (item) => item.id === id));
    }
    /**
     * @private
     * @param {?} collection
     * @param {?} id
     * @return {?}
     */
    indexOf(collection, id) {
        return collection.findIndex((/**
         * @param {?} item
         * @return {?}
         */
        (item) => item.id === id));
    }
    /**
     * @param {?} collectionName
     * @param {?} id
     * @return {?}
     */
    getInstance$(collectionName, id) {
        return new Observable((/**
         * @param {?} observer
         * @return {?}
         */
        (observer) => {
            /** @type {?} */
            const self = this;
            // Para testes e2e usa o banco de dados em memória mesmo
            if (this.config.memoryOnly) {
                /** @type {?} */
                const objectStore = this.dbe2e.get(collectionName);
                if (id !== undefined && id !== '') {
                    observer.next(this.findById(objectStore, id));
                    observer.complete();
                }
                else {
                    observer.error('Não foi passado o id');
                }
            }
            else {
                /** @type {?} */
                let request;
                /** @type {?} */
                const objectStore = self.db.transaction(collectionName, 'readwrite').objectStore(collectionName);
                if (id !== undefined && id !== '') {
                    request = objectStore.get(id);
                    request.onsuccess = (/**
                     * @param {?} event
                     * @return {?}
                     */
                    (event) => {
                        observer.next(request.result);
                        observer.complete();
                    });
                    request.onerror = (/**
                     * @param {?} event
                     * @return {?}
                     */
                    (event) => {
                        observer.error(((/** @type {?} */ (event.target))).error);
                    });
                }
                else {
                    observer.error('Não foi passado o id');
                }
            }
        }));
    }
    /**
     * @param {?} collectionName
     * @param {?} id
     * @param {?} query
     * @param {?} url
     * @param {?=} caseSensitiveSearch
     * @return {?}
     */
    get$(collectionName, id, query, url, caseSensitiveSearch) {
        return new Observable((/**
         * @param {?} observer
         * @return {?}
         */
        (observer) => {
            /** @type {?} */
            const self = this;
            /** @type {?} */
            let response;
            /** @type {?} */
            let request;
            /** @type {?} */
            let isCursor = false;
            /** @type {?} */
            let queryParams = { count: 0 };
            /** @type {?} */
            const queryResults = { hasNext: false, items: [] };
            /** @type {?} */
            const transformfn = self.transformGetMap.get(collectionName);
            // Para testes e2e usa o banco de dados em memória mesmo
            if (this.config.memoryOnly) {
                /** @type {?} */
                const objectStore = this.dbe2e.get(collectionName);
                if (id !== undefined && id !== '') {
                    /** @type {?} */
                    let item = this.findById(objectStore, id);
                    if (item && transformfn !== undefined) {
                        item = transformfn.call(this, item, this);
                    }
                    response = self.createResponseOptions(url, item ? STATUS.OK : STATUS.NOT_FOUND, item);
                    observer.next(response);
                    observer.complete();
                }
                else {
                    if (query) {
                        queryParams = self.getQueryParams(collectionName, query, (caseSensitiveSearch ? caseSensitiveSearch : 'i'));
                    }
                    /** @type {?} */
                    const cursor = {
                        index: 0,
                        value: null,
                        continue: (/**
                         * @return {?}
                         */
                        () => { })
                    };
                    while (cursor.index <= objectStore.length) {
                        cursor.value = (cursor.index < objectStore.length) ? objectStore[cursor.index++] : null;
                        if (self.getAllItems((cursor.value ? cursor : null), queryResults, queryParams, transformfn)) {
                            response = self.createResponseOptions(url, STATUS.OK, queryResults);
                            observer.next(response);
                            observer.complete();
                            break;
                        }
                    }
                }
            }
            else {
                /** @type {?} */
                const objectStore = self.db.transaction(collectionName, 'readwrite').objectStore(collectionName);
                if (id !== undefined && id !== '') {
                    request = objectStore.get(id);
                }
                else {
                    request = objectStore.openCursor();
                    isCursor = true;
                    if (query) {
                        queryParams = self.getQueryParams(collectionName, query, (caseSensitiveSearch ? caseSensitiveSearch : 'i'));
                    }
                }
                request.onsuccess = (/**
                 * @param {?} event
                 * @return {?}
                 */
                (event) => {
                    if (!isCursor) {
                        /** @type {?} */
                        let item = request.result;
                        if (item && transformfn !== undefined) {
                            item = transformfn.call(this, item, this);
                        }
                        response = self.createResponseOptions(url, item ? STATUS.OK : STATUS.NOT_FOUND, item);
                        observer.next(response);
                        observer.complete();
                    }
                    else {
                        if (self.getAllItems(((/** @type {?} */ (event.target))).result, queryResults, queryParams, transformfn)) {
                            response = self.createResponseOptions(url, STATUS.OK, queryResults);
                            observer.next(response);
                            observer.complete();
                        }
                    }
                });
                request.onerror = (/**
                 * @param {?} event
                 * @return {?}
                 */
                (event) => {
                    response = self.createErrorResponseOptions(url, STATUS.INTERNAL_SERVER_ERROR, ((/** @type {?} */ (event.target))).error);
                    observer.error(response);
                });
            }
        }));
    }
    /**
     * @param {?} reqInfo
     * @return {?}
     */
    get(reqInfo) {
        /** @type {?} */
        let response$;
        // Caso tenha um interceptador, retorna a resposta do mesmo
        response$ = this.processInterceptResponse(reqInfo, { id: reqInfo.id });
        if (response$) {
            return this.addDelay(response$, reqInfo.utils.getConfig().delay);
        }
        // Caso não tenha a collection, repassa a requisição para o backend verdadeiro
        // Para testes e2e usa o banco de dados em memória mesmo
        if (this.config.memoryOnly) {
            if (!this.dbe2e.has(reqInfo.collectionName)) {
                return undefined;
            }
        }
        else {
            if (!this.db.objectStoreNames.contains(reqInfo.collectionName)) {
                return undefined;
            }
        }
        response$ = this.get$(reqInfo.collectionName, reqInfo.id, reqInfo.query, reqInfo.url, reqInfo.utils.getConfig().caseSensitiveSearch ? undefined : 'i');
        return this.addDelay(response$, reqInfo.utils.getConfig().delay);
    }
    /**
     * @param {?} collectionName
     * @param {?} id
     * @param {?} item
     * @param {?} url
     * @return {?}
     */
    put$(collectionName, id, item, url) {
        return new Observable((/**
         * @param {?} observer
         * @return {?}
         */
        (observer) => {
            /** @type {?} */
            const self = this;
            /** @type {?} */
            let response;
            /** @type {?} */
            const transformfn = this.transformPutMap.get(collectionName);
            // Para testes e2e usa o banco de dados em memória mesmo
            if (this.config.memoryOnly) {
                /** @type {?} */
                const objectStore = this.dbe2e.get(collectionName);
                /** @type {?} */
                const existingIx = this.indexOf(objectStore, id);
                if (existingIx >= 0) {
                    if (transformfn !== undefined) {
                        item = transformfn.call(this, objectStore[existingIx], item);
                    }
                    item = Object.assign({}, objectStore[existingIx], item);
                    objectStore[existingIx] = item;
                    response = self.createResponseOptions(url, STATUS.NO_CONTENT);
                    observer.next(response);
                    observer.complete();
                }
                else {
                    response = self.createErrorResponseOptions(url, STATUS.NOT_FOUND, '\'' + collectionName + '\' item with id=\'' + id +
                        ' not found and may not be created with PUT; use POST instead.');
                    observer.error(response);
                }
            }
            else {
                /** @type {?} */
                const objectStore = self.db.transaction(collectionName, 'readwrite').objectStore(collectionName);
                /** @type {?} */
                const request = objectStore.get(id);
                request.onsuccess = (/**
                 * @param {?} event
                 * @return {?}
                 */
                (event) => {
                    if (request.result) {
                        if (transformfn !== undefined) {
                            item = transformfn.call(this, request.result, item);
                        }
                        item = Object.assign({}, request.result, item);
                        /** @type {?} */
                        const requestUpdate = objectStore.put(item);
                        requestUpdate.onsuccess = (/**
                         * @return {?}
                         */
                        () => {
                            response = self.createResponseOptions(url, STATUS.NO_CONTENT);
                            observer.next(response);
                            observer.complete();
                        });
                        requestUpdate.onerror = (/**
                         * @param {?} errorEvent
                         * @return {?}
                         */
                        (errorEvent) => {
                            response = self.createErrorResponseOptions(url, STATUS.INTERNAL_SERVER_ERROR, 'Erro ao atualizar \'' + collectionName + '\' pelo id: \'' +
                                id + '\'', ((/** @type {?} */ (errorEvent.target))).error);
                            observer.error(response);
                        });
                    }
                    else {
                        response = self.createErrorResponseOptions(url, STATUS.NOT_FOUND, '\'' + collectionName + '\' item with id=\'' + id +
                            ' not found and may not be created with PUT; use POST instead.');
                        observer.error(response);
                    }
                });
                request.onerror = (/**
                 * @param {?} event
                 * @return {?}
                 */
                (event) => {
                    response = self.createErrorResponseOptions(url, STATUS.INTERNAL_SERVER_ERROR, 'Erro ao buscar \'' + collectionName + '\' pelo id: \'' + id + '\'', ((/** @type {?} */ (event.target))).error);
                    observer.error(response);
                });
            }
        }));
    }
    /**
     * @param {?} reqInfo
     * @return {?}
     */
    put(reqInfo) {
        /** @type {?} */
        let response$;
        /** @type {?} */
        const item = this.clone(reqInfo.utils.getJsonBody(reqInfo.req)) || { id: reqInfo.id };
        if (!item.id) {
            item['id'] = reqInfo.id;
        }
        // Caso tenha um interceptador, retorna a resposta do mesmo
        response$ = this.processInterceptResponse(reqInfo, item);
        if (response$) {
            return this.addDelay(response$, reqInfo.utils.getConfig().delay);
        }
        // Caso não tenha a collection, repassa a requisição para o backend verdadeiro
        // Para testes e2e usa o banco de dados em memória mesmo
        if (this.config.memoryOnly) {
            if (!this.dbe2e.has(reqInfo.collectionName)) {
                return undefined;
            }
        }
        else {
            if (!this.db.objectStoreNames.contains(reqInfo.collectionName)) {
                return undefined;
            }
        }
        response$ = this.put$(reqInfo.collectionName, reqInfo.id, item, reqInfo.url);
        return this.addDelay(response$, reqInfo.utils.getConfig().delay);
    }
    /**
     * @param {?} collectionName
     * @param {?} item
     * @param {?} url
     * @return {?}
     */
    post$(collectionName, item, url) {
        return new Observable((/**
         * @param {?} observer
         * @return {?}
         */
        (observer) => {
            /** @type {?} */
            const self = this;
            /** @type {?} */
            let response;
            if (!item.id) {
                item['id'] = v4();
            }
            /** @type {?} */
            const transformfn = this.transformPostMap.get(collectionName);
            if (transformfn !== undefined) {
                item = transformfn.call(this, item, this);
            }
            // Para testes e2e usa o banco de dados em memória mesmo
            if (this.config.memoryOnly) {
                /** @type {?} */
                const objectStore = this.dbe2e.get(collectionName);
                objectStore.push(item);
                response = self.createResponseOptions(url, STATUS.CREATED, { id: item.id });
                response = response.clone({ headers: response.headers.append('Location', url + '/' + item.id) });
                observer.next(response);
                observer.complete();
            }
            else {
                /** @type {?} */
                const objectStore = self.db.transaction(collectionName, 'readwrite').objectStore(collectionName);
                /** @type {?} */
                const request = objectStore.add(item);
                request.onsuccess = (/**
                 * @param {?} event
                 * @return {?}
                 */
                (event) => {
                    if (request.result) {
                        response = self.createResponseOptions(url, STATUS.CREATED, { id: item.id });
                        response = response.clone({ headers: response.headers.append('Location', url + '/' + item.id) });
                        observer.next(response);
                        observer.complete();
                    }
                });
                request.onerror = (/**
                 * @param {?} event
                 * @return {?}
                 */
                (event) => {
                    response = self.createErrorResponseOptions(url, STATUS.INTERNAL_SERVER_ERROR, 'Erro ao incluir \'' + collectionName + '\' pelo id: \'' + item.id + '\'', ((/** @type {?} */ (event.target))).error);
                    observer.error(response);
                });
            }
        }));
    }
    /**
     * @param {?} reqInfo
     * @return {?}
     */
    post(reqInfo) {
        /** @type {?} */
        let response$;
        /** @type {?} */
        const item = this.clone(reqInfo.utils.getJsonBody(reqInfo.req)) || { id: reqInfo.id };
        if (!item.hasOwnProperty('id')) {
            item['id'] = reqInfo.id;
        }
        // Caso tenha um interceptador, retorna a resposta do mesmo
        response$ = this.processInterceptResponse(reqInfo, item);
        if (response$) {
            return this.addDelay(response$, reqInfo.utils.getConfig().delay);
        }
        // Caso não tenha a collection, repassa a requisição para o backend verdadeiro
        // Para testes e2e usa o banco de dados em memória mesmo
        if (this.config.memoryOnly) {
            if (!this.dbe2e.has(reqInfo.collectionName)) {
                return undefined;
            }
        }
        else {
            if (!this.db.objectStoreNames.contains(reqInfo.collectionName)) {
                return undefined;
            }
        }
        response$ = this.post$(reqInfo.collectionName, item, reqInfo.url);
        return this.addDelay(response$, reqInfo.utils.getConfig().delay);
    }
    /**
     * @param {?} collectionName
     * @param {?} id
     * @param {?} url
     * @return {?}
     */
    delete$(collectionName, id, url) {
        return new Observable((/**
         * @param {?} observer
         * @return {?}
         */
        (observer) => {
            /** @type {?} */
            const self = this;
            // Para testes e2e usa o banco de dados em memória mesmo
            if (this.config.memoryOnly) {
                /** @type {?} */
                const objectStore = this.dbe2e.get(collectionName);
                if (this.removeById(objectStore, id)) {
                    /** @type {?} */
                    const response = self.createResponseOptions(url, STATUS.NO_CONTENT, true);
                    observer.next(response);
                    observer.complete();
                }
                else {
                    /** @type {?} */
                    const response = self.createErrorResponseOptions(url, STATUS.NOT_FOUND, 'Erro ao buscar \'' + collectionName + '\' pelo id: \'' + id + '\'', 'Id não encontrado.');
                    observer.error(response);
                }
            }
            else {
                /** @type {?} */
                const objectStore = self.db.transaction(collectionName, 'readwrite').objectStore(collectionName);
                /** @type {?} */
                const request = objectStore.delete(id);
                request.onsuccess = (/**
                 * @param {?} event
                 * @return {?}
                 */
                (event) => {
                    /** @type {?} */
                    const response = self.createResponseOptions(url, STATUS.NO_CONTENT, true);
                    observer.next(response);
                    observer.complete();
                });
                request.onerror = (/**
                 * @param {?} event
                 * @return {?}
                 */
                (event) => {
                    /** @type {?} */
                    const response = self.createErrorResponseOptions(url, STATUS.INTERNAL_SERVER_ERROR, 'Erro ao buscar \'' + collectionName + '\' pelo id: \'' + id + '\'', ((/** @type {?} */ (event.target))).error);
                    observer.error(response);
                });
            }
        }));
    }
    /**
     * @param {?} reqInfo
     * @return {?}
     */
    delete(reqInfo) {
        /** @type {?} */
        let response$;
        // Caso tenha um interceptador, retorna a resposta do mesmo
        response$ = this.processInterceptResponse(reqInfo, { id: reqInfo.id });
        if (response$) {
            return this.addDelay(response$, reqInfo.utils.getConfig().delay);
        }
        // Caso não tenha a collection, repassa a requisição para o backend verdadeiro
        // Para testes e2e usa o banco de dados em memória mesmo
        if (this.config.memoryOnly) {
            if (!this.dbe2e.has(reqInfo.collectionName)) {
                return undefined;
            }
        }
        else {
            if (!this.db.objectStoreNames.contains(reqInfo.collectionName)) {
                return undefined;
            }
        }
        response$ = this.delete$(reqInfo.collectionName, reqInfo.id, reqInfo.url);
        return this.addDelay(response$, reqInfo.utils.getConfig().delay);
    }
    /**
     * @private
     * @param {?} collection
     * @param {?} id
     * @return {?}
     */
    removeById(collection, id) {
        /** @type {?} */
        const ix = this.indexOf(collection, id);
        if (ix > -1) {
            collection.splice(ix, 1);
            return true;
        }
        return false;
    }
    /**
     * @private
     * @param {?} interceptor
     * @param {?} uriInfo
     * @return {?}
     */
    compareRequestInterceptor(interceptor, uriInfo) {
        /** @type {?} */
        let interceptorPathOk = true;
        /** @type {?} */
        const intPaths = removeTrailingSlash(interceptor.path.trim()).split('/');
        /** @type {?} */
        const uriPaths = removeTrailingSlash(uriInfo.path).split('/');
        // Se possui o mesmo número de segmentos na URL
        interceptorPathOk = intPaths.length === uriPaths.length;
        if (interceptorPathOk) {
            // Avalia se todos os segmentos são iguais
            for (let i = 0; i < intPaths.length && interceptorPathOk; i++) {
                // Se é um segmento 'coriga' descarta o mesmo.
                // Utilizado para intercepatar urls tipo: /api/parent/:id/child/:id/action
                if (intPaths[i] === '**') {
                    continue;
                }
                interceptorPathOk = intPaths[i] === uriPaths[i];
            }
        }
        if (interceptorPathOk) {
            if (interceptor.query) {
                /** @type {?} */
                const interceptorQuery = (interceptor.query instanceof HttpParams) ? interceptor.query.toString() : interceptor.query;
                /** @type {?} */
                const uriInfoQuery = uriInfo.query ? decodeURI(uriInfo.query) : '';
                if (interceptorQuery.length < uriInfoQuery.length) {
                    return interceptorQuery === uriInfoQuery.substr(0, interceptorQuery.length);
                }
                else {
                    return uriInfoQuery.length ? uriInfoQuery === interceptorQuery.substr(0, uriInfoQuery.length) : true;
                }
            }
            else {
                return true;
            }
        }
        return false;
    }
    /**
     * @private
     * @return {?}
     */
    getResponseUtils() {
        return {
            responseFn: this.createResponseOptions.bind(this),
            errorResponseFn: this.createErrorResponseOptions.bind(this)
        };
    }
    /**
     * @private
     * @param {?} reqInfo
     * @param {?=} body
     * @return {?}
     */
    processInterceptResponse(reqInfo, body) {
        /** @type {?} */
        const response = this.interceptResponse(reqInfo, body);
        if (response instanceof Observable) {
            return response;
        }
        if (response !== undefined) {
            return new Observable((/**
             * @param {?} observer
             * @return {?}
             */
            observer => {
                if (response instanceof HttpErrorResponse) {
                    observer.error(response);
                }
                else {
                    observer.next(response);
                    observer.complete();
                }
            }));
        }
        return undefined;
    }
    /**
     * @private
     * @param {?} reqInfo
     * @param {?=} body
     * @return {?}
     */
    interceptResponse(reqInfo, body) {
        /** @type {?} */
        let response;
        /** @type {?} */
        const interceptor = reqInfo.utils['responseInterceptor'];
        if (interceptor) {
            if (reqInfo.method === interceptor.method.toLocaleLowerCase()) {
                if (interceptor.response) {
                    if (interceptor.response instanceof Function) {
                        response = interceptor.response.call(this, reqInfo.url, this.responseUtils, body);
                    }
                    else if (interceptor.response instanceof HttpResponse ||
                        interceptor.response instanceof HttpErrorResponse) {
                        response = this.clone(interceptor.response);
                    }
                }
            }
            delete reqInfo.utils['responseInterceptor'];
        }
        return response;
    }
    /**
     * @param {?} url
     * @param {?} reqInfoUtils
     * @return {?}
     */
    parseRequestUrl(url, reqInfoUtils) {
        console.log('[In-Memory]', 'Request: ', url);
        /** @type {?} */
        const self = this;
        /** @type {?} */
        const config = reqInfoUtils.getConfig();
        /** @type {?} */
        const start = config.host.length;
        for (const item of this.replaceMap.entries()) {
            if (url.includes(item[0], start)) {
                url = url.replace(item[0], item[1]);
                break;
            }
        }
        /** @type {?} */
        let parsed;
        try {
            parsed = reqInfoUtils.parseRequestUrl(url);
        }
        catch (err) {
            /** @type {?} */
            const msg = 'unable to parse url \'' + url + '\'; original error: ' + err.message;
            throw new Error(msg);
        }
        if (this.requestInterceptors.length > 0) {
            /** @type {?} */
            const uriInfo = reqInfoUtils.getLocation(url);
            /** @type {?} */
            const interceptor = this.requestInterceptors.find((/**
             * @param {?} value
             * @return {?}
             */
            (value) => {
                /** @type {?} */
                let intercept = value;
                if (value.path.indexOf('{0}') >= 0 && parsed.id) {
                    intercept = this.clone(value);
                    intercept.path = intercept.path.replace('{0}', parsed.id);
                }
                return self.compareRequestInterceptor(intercept, uriInfo);
            }));
            if (interceptor) {
                reqInfoUtils['responseInterceptor'] = interceptor;
            }
        }
        return parsed;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const dataServiceFn = new Map();
/** @type {?} */
const memoryConfig = {
    delay: 0,
    host: 'http://localhost:1080',
    apiBase: 'api/v1',
    passThruUnknownUrl: true,
    post204: false,
    put204: false,
    memoryOnly: false,
    databaseName: 'in-memory-database'
};
/**
 * @return {?}
 */
function getMemoryConfig() {
    return memoryConfig;
}
/**
 * @param {?} collectionName
 * @param {?} loadFn
 * @return {?}
 */
function dataService(collectionName, loadFn) {
    dataServiceFn.set(collectionName, loadFn);
}
/** @type {?} */
const dbService = new InMemoryDataService();
/**
 * @return {?}
 */
function getDbService() {
    return dbService;
}
/**
 * @param {?=} config
 * @return {?}
 */
function setupMemory(config) {
    if (config) {
        Object.assign(memoryConfig, config);
    }
    /** @type {?} */
    const result$ = new Promise((/**
     * @param {?} resolve
     * @param {?} reject
     * @return {?}
     */
    (resolve, reject) => {
        dbService.deleteDatabase().then((/**
         * @return {?}
         */
        () => {
            dbService.createDatabase().then((/**
             * @return {?}
             */
            () => {
                if (dataServiceFn && dataServiceFn.size > 0) {
                    dbService.createObjectStore(dataServiceFn).then((/**
                     * @return {?}
                     */
                    () => {
                        console.log('[In-Memory]', 'Database data created!');
                        resolve(true);
                    }), (/**
                     * @param {?} error
                     * @return {?}
                     */
                    error => {
                        console.error('[In-Memory]', error);
                        reject(error);
                    }));
                }
                else {
                    console.warn('[In-Memory]', 'There is not collection in data service!');
                    resolve(true);
                }
            }), (/**
             * @param {?} error
             * @return {?}
             */
            error => {
                console.error('[In-Memory]', error);
                reject(error);
            }));
        }), (/**
         * @param {?} error
         * @return {?}
         */
        error => {
            console.error('[In-Memory]', error);
            reject(error);
        }));
    }));
    return result$;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class InMemoryDataModule {
    /**
     * @return {?}
     */
    static forRoot() {
        return {
            ngModule: InMemoryDataModule,
            providers: [
                {
                    provide: InMemoryDbService,
                    useValue: getDbService()
                },
                {
                    provide: InMemoryBackendConfig,
                    useValue: getMemoryConfig()
                },
                {
                    provide: HttpBackend,
                    useFactory: httpClientInMemBackendServiceFactory,
                    deps: [
                        InMemoryDbService,
                        InMemoryBackendConfig,
                        XhrFactory
                    ]
                }
            ]
        };
    }
}
InMemoryDataModule.decorators = [
    { type: NgModule, args: [{},] }
];

export { InMemoryDataModule, InMemoryDataService, dataService, dataServiceFn, dbService, getDbService, getMemoryConfig, httpClientInMemBackendServiceFactory, memoryConfig, setupMemory };
//# sourceMappingURL=totvs-log-web-foundation-src-in-memory-data.js.map
