import { Component, Host, ElementRef, Renderer2, Input, NgModule, ChangeDetectorRef, Directive, Optional, Inject } from '@angular/core';
import { GridComponent, GridModule, PDFModule, ExcelModule } from '@progress/kendo-angular-grid';
import { PoPopupModule, PoTooltipDirective } from '@portinari/portinari-ui';
import { GlobalService } from 'totvs-log-web-foundation';
import { FormsModule } from '@angular/forms';
import { DropDownsModule } from '@progress/kendo-angular-dropdowns';
import { CommonModule, DOCUMENT } from '@angular/common';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class KendoGridExportComponent {
    /**
     * @param {?} grid
     * @param {?} el
     * @param {?} renderer
     */
    constructor(grid, el, renderer) {
        this.grid = grid;
        this.el = el;
        this.renderer = renderer;
        this.popupActionsExport = [
            {
                label: 'PDF',
                action: this.exportAsPDF
            },
            {
                label: 'Excel',
                action: this.exportAsExcel
            }
        ];
        this._posInGroup = false;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    set posInGroup(value) {
        this._posInGroup = convertToBoolean(value);
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        if (this._posInGroup) {
            this.renderer.removeChild(this.el.nativeElement, ((/** @type {?} */ (this.el.nativeElement))).children[0]);
            this.renderer.addClass(this.renderer.parentNode(this.el.nativeElement), 'k-grid-toolbar-hidden');
        }
    }
    /**
     * @return {?}
     */
    exportAsPDF() {
        this.grid.saveAsPDF();
    }
    /**
     * @return {?}
     */
    exportAsExcel() {
        this.grid.saveAsExcel();
    }
}
KendoGridExportComponent.decorators = [
    { type: Component, args: [{
                selector: 'lib-kendo-grid-export',
                template: "<span name=\"k-grid-export-space\">&nbsp;</span>\n<div class=\"k-grid-export\">\n  <span\n    #export\n    class=\"po-icon po-icon-export\"\n    p-tooltip=\"Exportar\"\n    p-tooltip-position=\"top\"\n    (click)=\"popup.toggle()\">\n  </span>\n  <po-popup\n    #popup\n    p-position=\"left\"\n    [p-target]=\"export\"\n    [p-actions]=\"popupActionsExport\">\n  </po-popup>\n</div>\n",
                styles: [""]
            }] }
];
/** @nocollapse */
KendoGridExportComponent.ctorParameters = () => [
    { type: GridComponent, decorators: [{ type: Host }] },
    { type: ElementRef },
    { type: Renderer2 }
];
KendoGridExportComponent.propDecorators = {
    posInGroup: [{ type: Input, args: ['pos-in-group',] }]
};
/**
 * @param {?} val
 * @return {?}
 */
function convertToBoolean(val) {
    if (typeof val === 'string') {
        val = val.toLowerCase().trim();
        return (val === 'true' || val === 'on' || val === '');
    }
    if (typeof val === 'number') {
        return val === 1;
    }
    return !!val;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class KendoGridExportModule {
}
KendoGridExportModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    KendoGridExportComponent
                ],
                imports: [
                    GridModule,
                    PoPopupModule
                ],
                exports: [
                    KendoGridExportComponent
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const flatten = (/**
 * @param {?} filter
 * @return {?}
 */
(filter) => {
    /** @type {?} */
    const filters = (filter || {}).filters;
    if (filters) {
        return filters.reduce((/**
         * @param {?} acc
         * @param {?} curr
         * @return {?}
         */
        (acc, curr) => acc.concat(curr.filters ? flatten(curr) : [curr])), []);
    }
    return [];
});
class KendoGridFilterEnumComponent {
    /**
     * @param {?} global
     */
    constructor(global) {
        this.global = global;
        this.options = [];
        this.selectedOptions = [];
    }
    /**
     * @return {?}
     */
    filterOptions() {
        this.selectedOptions.splice(0, this.selectedOptions.length, ...flatten(this.currentFilter).map((/**
         * @param {?} __0
         * @return {?}
         */
        ({ value }) => value)));
        return this.selectedOptions;
    }
    /**
     * @param {?} values
     * @return {?}
     */
    changeFiltro(values) {
        this.filterService.filter({
            filters: values.map((/**
             * @param {?} value
             * @return {?}
             */
            value => ({
                field: this.field,
                operator: this.applyFilter.bind(this),
                value,
                ignoreCase: false
            }))),
            logic: 'or'
        });
    }
    /**
     * @param {?} instance
     * @param {?} valor
     * @param {?} ignoreCase
     * @return {?}
     */
    applyFilter(instance, valor, ignoreCase) {
        /** @type {?} */
        const instanceValue = instance.hasOwnProperty('value') ? instance.value : instance.toString();
        return instanceValue === valor;
    }
}
KendoGridFilterEnumComponent.decorators = [
    { type: Component, args: [{
                selector: 'lib-kendo-grid-filter-enum',
                template: "<kendo-multiselect\n  [placeholder]=\"placeholder\"\n  [data]=\"options\"\n  textField=\"label\"\n  valueField=\"value\"\n  [valuePrimitive]=\"true\"\n  [value]=\"filterOptions()\"\n  (valueChange)=\"changeFiltro($event)\">\n  <kendo-multiselect-messages\n    [clearTitle]=\"global.i18n.literals['clearTitle']\"\n    [noDataText]=\"global.i18n.literals['nenhumDadoEncontrado']\">\n  </kendo-multiselect-messages>\n</kendo-multiselect>\n",
                styles: [""]
            }] }
];
/** @nocollapse */
KendoGridFilterEnumComponent.ctorParameters = () => [
    { type: GlobalService }
];
KendoGridFilterEnumComponent.propDecorators = {
    currentFilter: [{ type: Input }],
    filterService: [{ type: Input }],
    field: [{ type: Input }],
    placeholder: [{ type: Input }],
    options: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class KendoGridFilterEnumModule {
}
KendoGridFilterEnumModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    KendoGridFilterEnumComponent
                ],
                imports: [
                    GridModule,
                    FormsModule,
                    DropDownsModule
                ],
                exports: [
                    KendoGridFilterEnumComponent
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class KendoGridMessagesComponent {
    /**
     * @param {?} global
     * @param {?} changeDetectorRef
     */
    constructor(global, changeDetectorRef) {
        this.global = global;
        this.changeDetectorRef = changeDetectorRef;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        this.changeDetectorRef.detectChanges();
    }
}
KendoGridMessagesComponent.decorators = [
    { type: Component, args: [{
                selector: 'lib-kendo-grid-messages',
                template: "<kendo-grid-messages\n    [noRecords]=\"global.i18n.literals['nenhumDadoEncontrado']\"\n    [columnMenu]=\"global.i18n.literals['opcoesDaColuna']\"\n    [columns]=\"global.i18n.literals['colunaMenuGrid']\"\n    [filter]=\"global.i18n.literals['filtroMenuGrid']\"\n    [sortAscending]=\"global.i18n.literals['ordenarMenuGrid']\"\n    [sortDescending]=\"global.i18n.literals['ordenarMenuGrid']\"\n    [columnsApply]=\"global.i18n.literals['colunasAplicarMenuGrid']\"\n    [groupPanelEmpty]=\"global.i18n.literals['arrasteAColunaGridMessagem']\"\n    [columnsReset]=\"global.i18n.literals['colunasRestaurarMenuGrid']\"\n    [filterClearButton]=\"global.i18n.literals['colunasRestaurarMenuGrid']\"\n    [filterAfterOperator]=\"global.i18n.literals['depoisFiltroGrid']\"\n    [filterAfterOrEqualOperator]=\"global.i18n.literals['depoisOuIgualFiltroGrid']\"\n    [filterBooleanAll]=\"global.i18n.literals['todosBooleanFiltroGrid']\"\n    [filterContainsOperator]=\"global.i18n.literals['contemFiltroGrid']\"\n    [filterEndsWithOperator]=\"global.i18n.literals['terminaComFiltroGrid']\"\n    [filterAndLogic]=\"global.i18n.literals['eLogicoFiltroGrid']\"\n    [filterBeforeOperator]=\"global.i18n.literals['antesFiltroGrid']\"\n    [filterBeforeOrEqualOperator]=\"global.i18n.literals['anteriorOuIgualFiltroGrid']\"\n    [filterEqOperator]=\"global.i18n.literals['eIgualFiltroGrid']\"\n    [filterFilterButton]=\"global.i18n.literals['filtroGrid']\"\n    [filterGtOperator]=\"global.i18n.literals['eMelhorQueFiltroGrid']\"\n    [filterGteOperator]=\"global.i18n.literals['eMaiorQueOuIgualFiltroGrid']\"\n    [filterIsEmptyOperator]=\"global.i18n.literals['estaVaziaFiltroGrid']\"\n    [filterIsNotEmptyOperator]=\"global.i18n.literals['naoEstaVaziaFiltroGrid']\"\n    [filterIsNotNullOperator]=\"global.i18n.literals['naoNuloFiltroGrid']\"\n    [filterIsNullOperator]=\"global.i18n.literals['eNuloFiltroGrid']\"\n    [filterIsTrue]=\"global.i18n.literals['verdadeiroFiltroGrid']\"\n    [filterLtOperator]=\"global.i18n.literals['eMenosDoQueFIltroGrid']\"\n    [filterLteOperator]=\"global.i18n.literals['eMenorOuIgualFiltroGrid']\"\n    [filterNotContainsOperator]=\"global.i18n.literals['naoContemFiltroGrid']\"\n    [filterNotEqOperator]=\"global.i18n.literals['naoIgualFiltroGrid']\"\n    [filterOrLogic]=\"global.i18n.literals['ouLogicoFiltroGrid']\"\n    [filterStartsWithOperator]=\"global.i18n.literals['comecaComFiltroGrid']\">\n</kendo-grid-messages>\n",
                styles: [""]
            }] }
];
/** @nocollapse */
KendoGridMessagesComponent.ctorParameters = () => [
    { type: GlobalService },
    { type: ChangeDetectorRef }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class KendoGridMessagesModule {
}
KendoGridMessagesModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    KendoGridMessagesComponent
                ],
                imports: [
                    GridModule
                ],
                exports: [
                    KendoGridMessagesComponent
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ToolTipInKGridDirective {
    /**
     * @param {?} el
     * @param {?} tooltip
     */
    constructor(el, tooltip) {
        this.el = el;
        this.tooltip = tooltip;
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        ((/** @type {?} */ (this.el.nativeElement))).removeAllListeners('mouseenter');
        if (this.tooltip) {
            ((/** @type {?} */ (this.el.nativeElement))).addEventListener('mouseover', this.showTooltip.bind(this));
            // tslint:disable-next-line: no-string-literal
            this.tooltip['createTooltip']();
            this.tooltip.onMouseLeave();
        }
    }
    /**
     * @param {?} e
     * @return {?}
     */
    showTooltip(e) {
        /** @type {?} */
        const element = (/** @type {?} */ (e.target));
        if ((element.nodeName === 'TD' || element.nodeName === 'TH' ||
            element.parentNode.nodeName === 'TD' || element.parentNode.nodeName === 'TH')
            && element.offsetWidth < element.scrollWidth) {
            this.tooltip.tooltip = element.innerText;
            // tslint:disable-next-line: no-string-literal
            this.tooltip['poControlPosition'].setElements(this.tooltip['tooltipContent'], this.tooltip['tooltipOffset'], element);
            this.tooltip.onMouseEnter();
        }
        else {
            this.tooltip.onMouseLeave();
        }
    }
}
ToolTipInKGridDirective.decorators = [
    { type: Directive, args: [{
                selector: '[libToolTipInKGrid]'
            },] }
];
/** @nocollapse */
ToolTipInKGridDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: PoTooltipDirective, decorators: [{ type: Host }, { type: Optional }] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class TooltipInKGridModule {
}
TooltipInKGridModule.decorators = [
    { type: NgModule, args: [{
                declarations: [ToolTipInKGridDirective],
                imports: [
                    CommonModule,
                ],
                exports: [ToolTipInKGridDirective]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * \@dynamic
 */
class FixComboInKGridDirective {
    /**
     * @param {?} el
     * @param {?} document
     */
    constructor(el, document) {
        this.el = el;
        this.document = document;
    }
    /**
     * @return {?}
     */
    ngAfterViewChecked() {
        /** @type {?} */
        const container = this.el.nativeElement.querySelector('.po-combo-container');
        if (container) {
            this.document.body.appendChild(container);
        }
    }
}
FixComboInKGridDirective.decorators = [
    { type: Directive, args: [{
                selector: '[libFixComboInKGrid]'
            },] }
];
/** @nocollapse */
FixComboInKGridDirective.ctorParameters = () => [
    { type: ElementRef },
    { type: Document, decorators: [{ type: Inject, args: [DOCUMENT,] }] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class FixComboInKGridModule {
}
FixComboInKGridModule.decorators = [
    { type: NgModule, args: [{
                declarations: [FixComboInKGridDirective],
                imports: [
                    CommonModule,
                ],
                exports: [FixComboInKGridDirective]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
// @dynamic
class FndKendoGridModule {
}
FndKendoGridModule.decorators = [
    { type: NgModule, args: [{
                exports: [
                    GridModule,
                    PDFModule,
                    ExcelModule,
                    KendoGridMessagesModule,
                    KendoGridExportModule,
                    KendoGridFilterEnumModule,
                    TooltipInKGridModule,
                    FixComboInKGridModule
                ]
            },] }
];

export { FixComboInKGridDirective, FixComboInKGridModule, FndKendoGridModule, KendoGridExportComponent, KendoGridExportModule, KendoGridFilterEnumComponent, KendoGridFilterEnumModule, KendoGridMessagesComponent, KendoGridMessagesModule, ToolTipInKGridDirective, TooltipInKGridModule };
//# sourceMappingURL=totvs-log-web-foundation-src-kendo-grid.js.map
