import { Injectable, InjectionToken, Inject, Component, ViewChild, ComponentFactoryResolver, ApplicationRef, Injector, NgModule, EventEmitter, Input, Output, APP_INITIALIZER } from '@angular/core';
import { I18nBaseService, ShowMessageLinkActionType, AbstractShowMessage, ShowMessageType, MessageDisplayerService, LoadingDefault, DateTimeModule, AuthBaseService, CriticalError, AuthHandlerType, AUTH_SETTINGS, AUTH_TYPE, UserPermissionService, AuthInterceptorHandlerService } from 'totvs-log-base-foundation';
import { Observable } from 'rxjs';
import { map, mergeMap, catchError } from 'rxjs/operators';
import { HttpClient, HTTP_INTERCEPTORS } from '@angular/common/http';
import { PoI18nService, PoModalComponent, PoToasterOrientation, PoNotificationService, PoModule, PoModalModule, PoComponentsModule, PoI18nModule } from '@portinari/portinari-ui';
import { CommonModule } from '@angular/common';
import { FormsModule } from '@angular/forms';
import { NavigationEnd, Router, NavigationStart } from '@angular/router';
import { __awaiter } from 'tslib';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Serviço que realiza o carregamento das literais do serviço de internacionalização
 */
class IntlLiteralsLoaderService {
    /**
     * Método construtor da classe
     * @param {?} http Parâmetro do serviço de Http
     * @param {?} poI18nService Parâmetro do serviço de internacionalização do Po
     */
    constructor(http, poI18nService) {
        this.http = http;
        this.poI18nService = poI18nService;
        /**
         * Atributo com opções de internacionalização Web
         */
        this.webOptions = { context: '', language: '', url: '' };
    }
    /**
     * Método responsável por alterar as opções para carregamento de literais
     * @param {?} options Parâmetro com as opções de internacionalização (url, language, context)
     * @return {?}
     */
    updateOptions(options) {
        /** @type {?} */
        const obs = new Observable((/**
         * @param {?} observer
         * @return {?}
         */
        (observer) => {
            if (this.optionsWasChanged(options)) {
                Object.assign(this.webOptions, options);
                this.load().subscribe((/**
                 * @param {?} x
                 * @return {?}
                 */
                x => {
                    this.cache = x;
                    observer.next(this.cache);
                }));
            }
            else if (this.cache) {
                observer.next(this.cache);
            }
        }));
        return obs;
    }
    /**
     * Método responsável por realizar o carregamento inicial das literais
     * @return {?}
     */
    load() {
        return this.http.get(this.webOptions.url)
            .pipe(map((/**
         * @param {?} res
         * @return {?}
         */
        (res) => {
            /** @type {?} */
            const s = (/** @type {?} */ (this.poI18nService));
            s.config.contexts[this.webOptions.context] = res;
            s.setVarI18n(s.config.contexts);
            return res;
        })), mergeMap((/**
         * @param {?} res
         * @return {?}
         */
        (res) => this.updateLiteralsFromPoContext())), catchError((/**
         * @param {?} e
         * @param {?} caught
         * @return {?}
         */
        (e, caught) => caught)));
    }
    /**
     * Detecta se ocorreram mudanças na configuração
     * @private
     * @param {?} options
     * @return {?}
     */
    optionsWasChanged(options) {
        return this.webOptions.context !== options.context
            || this.webOptions.url !== options.url
            || this.webOptions.language !== options.language;
    }
    /**
     * Método responsável por carregar as literais do contexto do Po
     * @private
     * @return {?}
     */
    updateLiteralsFromPoContext() {
        return this.poI18nService.getLiterals({
            context: this.webOptions.context,
            language: this.webOptions.language
        }).pipe(map((/**
         * @param {?} res
         * @return {?}
         */
        (res) => res)));
    }
}
IntlLiteralsLoaderService.decorators = [
    { type: Injectable }
];
/** @nocollapse */
IntlLiteralsLoaderService.ctorParameters = () => [
    { type: HttpClient },
    { type: PoI18nService }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Token de configuração da url de internacionalização
 * @type {?}
 */
const INTL_CONFIG = new InjectionToken('Configurações da internacionalização');
/**
 * Serviço de internacionalização
 */
class IntlService extends I18nBaseService {
    /**
     * Método construtor da classe
     * @param {?} config Parâmetro do serviço com as configurações de internacionalização
     * @param {?} intlLiteralsLoader Parâmetro do serviço de carregamento de literais de internacionalização
     */
    constructor(config, intlLiteralsLoader) {
        super(intlLiteralsLoader);
        /** @type {?} */
        let matchedAppLanguage;
        if (config.supportedLanguages) {
            matchedAppLanguage = config.supportedLanguages.find((/**
             * @param {?} appSupportedLanguage
             * @return {?}
             */
            (appSupportedLanguage) => {
                return appSupportedLanguage.language === navigator.language;
            }));
            matchedAppLanguage = matchedAppLanguage ? matchedAppLanguage : IntlService.i18nConfig.default.language;
        }
        else {
            matchedAppLanguage = IntlService.i18nConfig.default.language;
            config.supportedLanguages = [{
                    language: IntlService.i18nConfig.default.language
                }];
        }
        this.updateOptions({
            context: IntlService.i18nConfig.default.context,
            language: matchedAppLanguage.language,
            url: config.url,
            supportedLanguages: config.supportedLanguages
        });
    }
}
/**
 * Atributo com as configurações de internacionalização
 */
IntlService.i18nConfig = {
    contexts: {},
    default: {
        language: 'pt-br',
        context: 'general',
        cache: true
    }
};
IntlService.decorators = [
    { type: Injectable }
];
/** @nocollapse */
IntlService.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [INTL_CONFIG,] }] },
    { type: IntlLiteralsLoaderService }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Classe/componente de mensagem
 */
class MessageComponent {
    constructor() {
        /**
         * Atributo com o título da mensagem
         */
        this.messageTitle = '';
        /**
         * Atributo com o texto da mensagem
         */
        this.messageText = '';
        /**
         * Atributo com os detalhes da mensagem
         */
        this.messageDetails = '';
        /**
         * Atributo que recebe a ação de confirmação
         */
        this.messageConfirmAction = undefined;
        /**
         * Atributo que recebe a ação de cancelamento
         */
        this.messageCancelAction = undefined;
        /**
         * Atributo que recebe o tipo do link
         */
        this.messageLinkActionType = ShowMessageLinkActionType.No;
        /**
         * Atributo que recebe o link
         */
        this.messageLinkAction = undefined;
        /**
         * Atributo que recebe o label do link
         */
        this.messageLinkLabel = '';
        /**
         * Atributo que indica se os detalhes da mensagem devem ser escondidos
         */
        this.hideDetails = true;
    }
    /**
     * Método responsável por disparar a abertura da dialog
     * @return {?}
     */
    openDialog() {
        this.poModal.open();
    }
    /**
     * Método responsável por fechar a dialog
     * @return {?}
     */
    closeDialog() {
        this.poModal.close();
    }
    /**
     * Método responsável por executar ação do link relacionado a mensagem
     * @return {?}
     */
    linkAction() {
        this.messageLinkAction();
    }
}
MessageComponent.decorators = [
    { type: Component, args: [{
                selector: 'lib-message',
                template: "<po-modal\n  [p-title]=\"messageTitle\"\n  [p-primary-action]=\"messageConfirmAction\"\n  [p-secondary-action]=\"messageCancelAction\">\n  <div class=\"message-body\">\n    {{messageText}}\n    <p *ngIf=\"messageLinkActionType\">\n      <a href=\"javascript:void(0)\" (click)=\"linkAction()\">{{messageLinkLabel}}</a>\n    </p>\n    <br/>\n    <div *ngIf=\"messageDetails\" class=\"message-details\">\n      <a *ngIf=\"hideDetails\" class=\"show-details\" href=\"javascript:void(0);\" (click)=\"hideDetails = !hideDetails\">\n        <span class=\"po-icon po-icon-arrow-down\"></span>\n        Exibir detalhes\n      </a>\n      <a *ngIf=\"!hideDetails\" class=\"show-details\" href=\"javascript:void(0);\" (click)=\"hideDetails = !hideDetails\">\n        <span class=\"po-icon po-icon-arrow-up\"></span>\n        Ocultar detalhes\n      </a>\n      <p [ngClass]=\"{'hide-details': hideDetails}\">\n        {{messageDetails}}\n      </p>\n    </div>    \n  </div>\n</po-modal>",
                styles: [".show-details{height:12px;font-family:NunitoSans-Bold;font-size:14px!important;line-height:1.71;text-align:left;color:#1d9abc;padding-top:13px;padding-bottom:15px;text-decoration:none}.hide-details{display:none}.message-body{overflow:hidden;margin-bottom:4px;margin-left:10px;vertical-align:top;min-height:45px}.message-details{margin-top:15px;text-align:center;border-top:1px solid #dbdedf}"]
            }] }
];
MessageComponent.propDecorators = {
    poModal: [{ type: ViewChild, args: [PoModalComponent, { static: true },] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Serviço de mensagens
 */
class MessageService extends AbstractShowMessage {
    /**
     * Método construtor da classe
     * @param {?} poNotification Parâmetro do serviço de notificação do Po
     * @param {?} componentFactoryResolver Parâmetro do serviço que permite adicionar componentes dinamicamente
     * @param {?} appRef Parâmetro que recebe a referência da aplicação Angular.
     * @param {?} injector Parâmetro do serviço de injeção de dependência
     * @param {?} i18n  Parâmetro do serviço de internacionalização
     */
    constructor(poNotification, componentFactoryResolver, appRef, injector, i18n) {
        super();
        this.poNotification = poNotification;
        this.componentFactoryResolver = componentFactoryResolver;
        this.appRef = appRef;
        this.injector = injector;
        this.i18n = i18n;
        /**
         * Atributo que recebe a instância da dialog criada
         */
        this.messageDialogRef = undefined;
    }
    /**
     * Método responsável por exibir uma mensagem, através de notificação ou dialog
     * @param {?} showMessageOptions Parâmetro com as definições da mensagem
     * @return {?}
     */
    displayMessage(showMessageOptions) {
        showMessageOptions = showMessageOptions;
        switch (showMessageOptions.messageType) {
            case ShowMessageType.NotificationError:
            case ShowMessageType.NotificationInformation:
            case ShowMessageType.NotificationSuccess:
            case ShowMessageType.NotificationWarning:
                this.showNotification(showMessageOptions);
                break;
            case ShowMessageType.DialogAlert:
            case ShowMessageType.DialogConfirmation:
                this.showDialog(showMessageOptions);
                break;
            default:
                throw new Error(`Tipo de mensagem inválido: ${showMessageOptions.messageType}`);
        }
    }
    /**
     * Método responsável por exibir uma notificação em tela
     * @private
     * @param {?} showMessageOptions Parâmetro com as definições da mensagem
     * @return {?}
     */
    showNotification(showMessageOptions) {
        /** @type {?} */
        const poNotification = {
            message: showMessageOptions.messageText,
            orientation: PoToasterOrientation.Bottom,
            action: this.processLinkAction(showMessageOptions),
            actionLabel: showMessageOptions.messageLinkLabel
        };
        switch (showMessageOptions.messageType) {
            case ShowMessageType.NotificationSuccess:
                this.poNotification.success(poNotification);
                break;
            case ShowMessageType.NotificationError:
                this.poNotification.error(poNotification);
                break;
            case ShowMessageType.NotificationInformation:
                this.poNotification.information(poNotification);
                break;
            case ShowMessageType.NotificationWarning:
                this.poNotification.warning(poNotification);
                break;
        }
    }
    /**
     * Método por exibir uma dialog em tela
     * @private
     * @param {?} showMessageOptions Parâmetro com as definições da mensagem
     * @return {?}
     */
    showDialog(showMessageOptions) {
        this.createDialogComponent();
        this.setDialogOptions(showMessageOptions);
        this.messageDialogRef.instance.openDialog();
    }
    /**
     * Método responsável por abrir no navegador uma nova tab com o link recebido por parâmetro
     * @private
     * @param {?} link Link de uma página externa
     * @return {?}
     */
    openLink(link) {
        link = link.indexOf('http://') === -1 ? `http://${link}` : link;
        window.open(link, '_blank');
    }
    /**
     * Método responsável por identificar se o link recebido é uma função ou página externa
     * @private
     * @param {?} showMessageOptions Parâmetro com as definições da mensagem
     * @return {?}
     */
    processLinkAction(showMessageOptions) {
        /** @type {?} */
        let action;
        if (showMessageOptions.messageLinkActionType === ShowMessageLinkActionType.Method) {
            action = showMessageOptions.messageLinkAction;
        }
        else if (showMessageOptions.messageLinkActionType === ShowMessageLinkActionType.LinkNewTab &&
            showMessageOptions.messageLinkAction && showMessageOptions.messageLinkAction.trim() !== '') {
            action = (/**
             * @return {?}
             */
            () => {
                this.openLink(showMessageOptions.messageLinkAction);
            });
        }
        return action;
    }
    /**
     * Método responsável por criar o componente dialog
     * @private
     * @return {?}
     */
    createDialogComponent() {
        if (this.messageDialogRef === undefined) {
            this.messageDialogRef = this.componentFactoryResolver
                .resolveComponentFactory(MessageComponent)
                .create(this.injector);
            this.appRef.attachView(this.messageDialogRef.hostView);
            /** @type {?} */
            const domElem = (/** @type {?} */ (((/** @type {?} */ (this.messageDialogRef.hostView)))
                .rootNodes[0]));
            document.body.appendChild(domElem);
        }
    }
    /**
     * Método responsável por setar as opções do componente dialog
     * @private
     * @param {?} showMessageOptions Parâmetro com as definições da mensagem
     * @return {?}
     */
    setDialogOptions(showMessageOptions) {
        if (showMessageOptions.messageTitle !== undefined &&
            showMessageOptions.messageTitle.trim() !== '') {
            this.messageDialogRef.instance.messageTitle = showMessageOptions.messageTitle;
        }
        else if (showMessageOptions.messageType === ShowMessageType.DialogAlert) {
            this.messageDialogRef.instance.messageTitle = this.i18n.literals.warning;
        }
        else if (showMessageOptions.messageType === ShowMessageType.DialogConfirmation) {
            this.messageDialogRef.instance.messageTitle = this.i18n.literals.confirm;
        }
        this.messageDialogRef.instance.messageText = showMessageOptions.messageText;
        this.messageDialogRef.instance.messageDetails = showMessageOptions.messageDetails;
        this.messageDialogRef.instance.messageConfirmAction = {
            action: (/**
             * @return {?}
             */
            () => {
                if (showMessageOptions.messageConfirmAction) {
                    showMessageOptions.messageConfirmAction();
                }
                this.messageDialogRef.instance.closeDialog();
            }),
            label: showMessageOptions.messageConfirmLabel
        };
        this.messageDialogRef.instance.messageCancelAction = {
            action: (/**
             * @return {?}
             */
            () => {
                if (showMessageOptions.messageCancelAction) {
                    showMessageOptions.messageCancelAction();
                }
                this.messageDialogRef.instance.closeDialog();
            }),
            label: showMessageOptions.messageCancelLabel
        };
        this.messageDialogRef.instance.messageLinkActionType = showMessageOptions.messageLinkActionType;
        if (showMessageOptions.messageLinkActionType === ShowMessageLinkActionType.LinkNewTab) {
            /** @type {?} */
            const link = showMessageOptions.messageLinkAction;
            showMessageOptions.messageLinkAction = (/**
             * @return {?}
             */
            () => {
                this.openLink(link);
            });
        }
        this.messageDialogRef.instance.messageLinkAction = showMessageOptions.messageLinkAction;
        this.messageDialogRef.instance.messageLinkLabel = showMessageOptions.messageLinkLabel;
    }
}
MessageService.decorators = [
    { type: Injectable }
];
/** @nocollapse */
MessageService.ctorParameters = () => [
    { type: PoNotificationService },
    { type: ComponentFactoryResolver },
    { type: ApplicationRef },
    { type: Injector },
    { type: I18nBaseService }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class MessageModule {
}
MessageModule.decorators = [
    { type: NgModule, args: [{
                declarations: [MessageComponent],
                imports: [
                    CommonModule,
                    PoModule
                ],
                exports: [],
                providers: [
                    MessageService,
                    MessageDisplayerService,
                    {
                        provide: AbstractShowMessage,
                        useClass: MessageService
                    }
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class ChangeLanguageComponent {
    /**
     * @param {?} i18n
     */
    constructor(i18n) {
        this.i18n = i18n;
        this.literals = {
            modalTitle: 'changeLanguage',
            radioGroupLabel: 'selectLanguage',
            saveAction: 'save',
            cancelAction: 'cancel'
        };
        this.afterChangeLanguage = new EventEmitter();
        this._languageOptions = [];
    }
    /**
     * @return {?}
     */
    get primaryAction() {
        return this._primaryAction;
    }
    /**
     * @return {?}
     */
    get secondaryAction() {
        return this._secondaryAction;
    }
    /**
     * @return {?}
     */
    get languageOptions() {
        return this._languageOptions;
    }
    /**
     * @return {?}
     */
    ngOnInit() { }
    /**
     * @return {?}
     */
    openChangeLanguageModal() {
        this.setModalActions();
        this.setLanguageOptions();
        this.selectedLanguage = this.i18n.getLanguage();
        this.changeLanguageModal.open();
    }
    /**
     * @private
     * @return {?}
     */
    setModalActions() {
        this._primaryAction = {
            label: this.i18n.literals[this.literals.saveAction],
            action: (/**
             * @return {?}
             */
            () => {
                this.changeLanguage();
            })
        };
        this._secondaryAction = {
            label: this.i18n.literals[this.literals.cancelAction],
            action: (/**
             * @return {?}
             */
            () => {
                this.changeLanguageModal.close();
            })
        };
    }
    /**
     * @private
     * @return {?}
     */
    setLanguageOptions() {
        this._languageOptions = [];
        this.i18n.getSupportedLanguages().forEach((/**
         * @param {?} supportedLanguage
         * @return {?}
         */
        supportedLanguage => {
            this._languageOptions.push({
                label: this.i18n.literals[supportedLanguage.language],
                value: supportedLanguage.language
            });
        }));
    }
    /**
     * @private
     * @return {?}
     */
    changeLanguage() {
        /** @type {?} */
        const subscribe = this.i18n.updateOptions({ language: this.selectedLanguage }).subscribe((/**
         * @return {?}
         */
        () => {
            this.afterChangeLanguage.emit(this.selectedLanguage);
            this.changeLanguageModal.close();
        }), (/**
         * @return {?}
         */
        () => { }), (/**
         * @return {?}
         */
        () => {
            subscribe.unsubscribe();
        }));
    }
}
ChangeLanguageComponent.decorators = [
    { type: Component, args: [{
                selector: 'lib-change-language',
                template: "<po-modal #changeLanguageModal\n  [p-title]=\"i18n.literals[literals.modalTitle]\"\n  p-click-out=\"true\"\n  p-size=\"sm\"\n  [p-primary-action]=\"primaryAction\"\n  [p-secondary-action]=\"secondaryAction\">\n  <po-radio-group\n    name=\"radioGroupLanguage\"\n    [(ngModel)]=\"selectedLanguage\"\n    [p-label]=\"i18n.literals[literals.radioGroupLabel]\"\n    [p-options]=\"languageOptions\"\n    p-required\n    p-columns=\"1\">\n  </po-radio-group>\n</po-modal>",
                styles: [""]
            }] }
];
/** @nocollapse */
ChangeLanguageComponent.ctorParameters = () => [
    { type: I18nBaseService }
];
ChangeLanguageComponent.propDecorators = {
    literals: [{ type: Input }],
    afterChangeLanguage: [{ type: Output }],
    changeLanguageModal: [{ type: ViewChild, args: [PoModalComponent, { static: true },] }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class IntlModule {
}
IntlModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    FormsModule,
                    PoModalModule,
                    PoComponentsModule,
                    PoI18nModule.config(IntlService.i18nConfig)
                ],
                exports: [
                    PoI18nModule,
                    ChangeLanguageComponent
                ],
                providers: [
                    IntlService,
                    IntlLiteralsLoaderService,
                    {
                        provide: I18nBaseService,
                        useClass: IntlService
                    }
                ],
                declarations: [
                    ChangeLanguageComponent
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Componenente de loading que utiliza o componente padrão do PO
 */
class LoadingComponent {
    constructor() {
        /**
         * Texto do loading
         */
        this.text = '';
        /**
         * Definine se será bloqueada a tela inteira ou apenas o container em que o componente de loading está inserido
         */
        this.screenLock = false;
        /**
         * Define se o loading está sendo mostrado ou não
         */
        this.show = false;
    }
    /**
     * Permite atribuir o texto do loading
     * @param {?=} text
     * @return {?}
     */
    setLoadingText(text = '') {
        this.text = text;
    }
    /**
     * Apresenta o loading, bloqueando a tela
     * @return {?}
     */
    showLoading() {
        this.show = true;
    }
    /**
     * Esconde o loading, desbloqueando a tela
     * @return {?}
     */
    hideLoading() {
        this.show = false;
    }
}
LoadingComponent.decorators = [
    { type: Component, args: [{
                selector: 'lib-loading',
                template: "<po-loading-overlay\n    *ngIf=\"show\"\n    [p-screen-lock]=\"screenLock\"\n    [p-text]=\"text\">\n</po-loading-overlay>",
                styles: [""]
            }] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class LoadingService extends LoadingDefault {
    /**
     * Construtor do serviço
     * @param {?} i18n Serviço de internacionalização
     * @param {?} componentFactoryResolver Serviço que permite adicionar componentes dinamicamente
     * @param {?} appRef Referência da aplicação Angular
     * @param {?} injector Serviço de injeção de dependência
     */
    constructor(i18n, componentFactoryResolver, appRef, injector) {
        super();
        this.i18n = i18n;
        this.componentFactoryResolver = componentFactoryResolver;
        this.appRef = appRef;
        this.injector = injector;
        /**
         * Referência para o componente de loading no html
         */
        this.loadingComponentRef = undefined;
        /**
         * Texto apresentado durante o loading
         */
        this.loadingText = '';
    }
    /**
     * Exibe o loading, bloqueando a tela
     * @return {?}
     */
    showLoading() {
        if (this.loadingComponentRef === undefined) {
            this.createLoadingComponent();
        }
        this.loadingComponentRef.instance.showLoading();
    }
    /**
     * Esconde o loading, desbloqueando a tela
     * @return {?}
     */
    hideLoading() {
        if (this.loadingComponentRef !== undefined) {
            this.loadingComponentRef.instance.hideLoading();
        }
    }
    /**
     * Atribui o texto que será apresentado
     * @param {?} text Texto a ser apresentado, devendo ser um BaseMessage
     * @return {?}
     */
    setLoadingText(text) {
        this.loadingText = this.i18n.text(text) ? this.i18n.text(text).text : '';
        if (this.loadingComponentRef === undefined) {
            this.createLoadingComponent();
        }
        this.loadingComponentRef.instance.text = this.loadingText;
    }
    /**
     * Método privado que cria dinamicamente o componente de loading no DOM
     * @private
     * @return {?}
     */
    createLoadingComponent() {
        if (this.loadingComponentRef === undefined) {
            this.loadingComponentRef = this.componentFactoryResolver
                .resolveComponentFactory(LoadingComponent)
                .create(this.injector);
            this.appRef.attachView(this.loadingComponentRef.hostView);
            /** @type {?} */
            const domElem = (/** @type {?} */ (((/** @type {?} */ (this.loadingComponentRef.hostView)))
                .rootNodes[0]));
            document.body.appendChild(domElem);
        }
    }
}
LoadingService.decorators = [
    { type: Injectable }
];
/** @nocollapse */
LoadingService.ctorParameters = () => [
    { type: I18nBaseService },
    { type: ComponentFactoryResolver },
    { type: ApplicationRef },
    { type: Injector }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Serviço responsável por métodos e atributos utilizados por toda aplicação web
 */
class GlobalService {
    /**
     * Método construtor que inicializa a classe
     * @param {?} i18n Parâmetro do serviço de internacionalização
     * @param {?} msg  Parâmetro do serviço de mensagens
     * @param {?} loading Parâmetro do serviço de loading
     */
    constructor(i18n, msg, loading) {
        this.i18n = i18n;
        this.msg = msg;
        this.loading = loading;
    }
}
GlobalService.decorators = [
    { type: Injectable }
];
/** @nocollapse */
GlobalService.ctorParameters = () => [
    { type: I18nBaseService },
    { type: MessageDisplayerService },
    { type: LoadingService }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class LoadingModule {
}
LoadingModule.decorators = [
    { type: NgModule, args: [{
                declarations: [LoadingComponent],
                imports: [
                    CommonModule,
                    PoModule
                ],
                exports: [],
                providers: [
                    LoadingService
                ]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const i18nFactory = (/**
 * @param {?} i18n
 * @param {?} config
 * @return {?}
 */
(i18n, config) => {
    return (/**
     * @return {?}
     */
    () => new Promise((/**
     * @param {?} resolve
     * @return {?}
     */
    resolve => {
        if (config.lazy) {
            resolve(true);
        }
        else {
            i18n.updateOptions({}).subscribe((/**
             * @return {?}
             */
            () => {
                resolve(true);
            }), (/**
             * @param {?} error
             * @return {?}
             */
            (error) => {
                console.error(error);
                resolve(false);
            }));
        }
    })));
});

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Módulo que exporta e ativas as dependências basicas do foundation
 * Para garantir que os serviços tenham uma única instância, usar FndModule.forRoot() ao importar
 */
class FndModule {
    /**
     * @return {?}
     */
    static forRoot() {
        return {
            ngModule: FndModule,
            providers: [
                GlobalService,
                {
                    provide: APP_INITIALIZER,
                    useFactory: i18nFactory,
                    deps: [I18nBaseService, INTL_CONFIG],
                    multi: true
                }
            ]
        };
    }
}
FndModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    PoModule,
                    IntlModule,
                    MessageModule,
                    DateTimeModule,
                    LoadingModule
                ],
                exports: [
                    IntlModule,
                    MessageModule,
                    DateTimeModule,
                    LoadingModule
                ],
                entryComponents: [MessageComponent, LoadingComponent]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Representa um componente para aceitar o redirecionamento de callback do fluxo de oauth2
 */
class AuthCallbackComponent {
    /**
     * Creates an instance of AuthCallbackComponent.
     * @param {?} router
     * @param {?} authService
     */
    constructor(router, authService) {
        this.router = router;
        this.authService = authService;
        this.router.events.forEach((/**
         * @param {?} event
         * @return {?}
         */
        event => {
            if (event instanceof NavigationEnd) {
                this.authService
                    .completeAuthentication()
                    .then((/**
                 * @return {?}
                 */
                () => {
                    this.authService
                        .getUser()
                        .then((/**
                     * @param {?} user
                     * @return {?}
                     */
                    (user) => {
                        this.authService.userLoadedEvent.emit(user);
                    }));
                }));
            }
        }));
    }
    /**
     * @return {?}
     */
    ngOnInit() {
    }
}
AuthCallbackComponent.decorators = [
    { type: Component, args: [{
                selector: 'lib-auth-callback',
                template: ''
            }] }
];
/** @nocollapse */
AuthCallbackComponent.ctorParameters = () => [
    { type: Router },
    { type: AuthBaseService }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Implementação de guard com uso do serviço de autenticação padrão
 */
class AuthGuard {
    /**
     * Creates an instance of AuthGuard.
     * @param {?} authService
     */
    constructor(authService) {
        this.authService = authService;
    }
    /**
     * Implementação do serviço angular que previne o carregamento de componentes, se necessário
     * @return {?}
     */
    canActivate() {
        return __awaiter(this, void 0, void 0, function* () {
            /** @type {?} */
            const isAuthorized = yield this.authService.authorize();
            if (isAuthorized) {
                /** @type {?} */
                const user = yield this.authService.getUser();
                this.authService.userLoadedEvent.next(user);
            }
            return isAuthorized;
        });
    }
}
AuthGuard.decorators = [
    { type: Injectable }
];
/** @nocollapse */
AuthGuard.ctorParameters = () => [
    { type: AuthBaseService }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Classe de serviço com métodos para autenticação
 * @extends AuthBaseService
 */
class AuthService extends AuthBaseService {
    /**
     * Método construtor da classe
     * @param {?} authSettings Token de injeção com as configurações de autenticação
     * @param {?} authType Token de injeção com o tipo de autenticação
     */
    constructor(authSettings, authType) {
        super(AuthService.AddOriginUrls(authSettings));
        this.authSettings = authSettings;
        this.authType = authType;
        if (this.authSettings === undefined) {
            throw new CriticalError('authSettings: undefined');
        }
        if (this.authType === undefined) {
            throw new CriticalError('authType: undefined');
        }
    }
    /**
     * Adiciona nas uris de redirecionamento do processo oauth o host de origem, se um host não for informado
     * Isso é para permitir que o navegador identifique o domínio de redirecionamento
     * utilizando o caminho relativo
     *
     * @param {?} authSettings as configurações de autenticação
     * @return {?} as configurações de autenticação com uris normalizadas para o processo de autenticação
     */
    static AddOriginUrls(authSettings) {
        if (authSettings) {
            authSettings.redirect_uri = AuthService.AddOriginUrl(window.location.origin, authSettings.redirect_uri);
            authSettings.post_logout_redirect_uri
                = AuthService.AddOriginUrl(window.location.origin, authSettings.post_logout_redirect_uri);
            authSettings.silent_redirect_uri = AuthService.AddOriginUrl(window.location.origin, authSettings.silent_redirect_uri);
            if (!authSettings.post_logout_redirect_uri) {
                authSettings.post_logout_redirect_uri = window.location.origin;
            }
        }
        return authSettings;
    }
    /**
     * Adiciona o host na url, se nenhum host estiver presente
     * @param {?} host o host a ser adicionado
     * @param {?} url url em que o host será adicionado
     * @return {?} a junção da url e host
     */
    static AddOriginUrl(host, url) {
        if (url.trim().length !== 0 && !url.startsWith(host) && !url.startsWith('http')) {
            url = `${host}${url}`;
        }
        return url;
    }
    /**
     * Verifica se está em atuação o fluxo de login e deveria ou não mostrar o menu
     * @param {?} router
     * @return {?}
     */
    shouldShowMenu(router) {
        /** @type {?} */
        let loginFlow = false;
        router.events.forEach((/**
         * @param {?} event
         * @return {?}
         */
        (event) => {
            if (event instanceof NavigationStart) {
                loginFlow = event.url === '' || event.url === '/' || event.url.startsWith('/auth-callback');
                // vai redirecionar, guarda a rota atual
                if (!loginFlow) {
                    localStorage.setItem('current.route', event.url);
                }
            }
        }));
        return loginFlow;
    }
    /**
     * Monta um PoToolbarProfile a partir do usuário. Sem suporte a avatar no momento
     * @param {?} user
     * @return {?}
     */
    getToolbarProfile(user) {
        return {
            title: user && user.profile ? user.profile.name : null,
            subtitle: user && user.profile ? user.profile.email : null
        };
    }
    /**
     * Método responsável executar o processo de login/autenticação
     * @return {?}
     */
    loginFlow() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.authType.type === AuthHandlerType.oauth2) {
                yield this.signinRedirect();
            }
        });
    }
    /**
     * Método responsável executar o processo de logout
     * @return {?}
     */
    logoutFlow() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.authType.type === AuthHandlerType.oauth2) {
                yield this.signoutRedirect();
            }
        });
    }
}
AuthService.decorators = [
    { type: Injectable }
];
/** @nocollapse */
AuthService.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: [AUTH_SETTINGS,] }] },
    { type: undefined, decorators: [{ type: Inject, args: [AUTH_TYPE,] }] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Módulo que exporta e ativas as dependências para utilizar autenticação com fluxo oauth2
 * Para garantir que os serviços tenham uma única instância, usar AuthModule.forRoot() ao importar
 */
class AuthModule {
    /**
     * @return {?}
     */
    static forRoot() {
        return {
            ngModule: AuthModule,
            providers: [
                AuthService,
                UserPermissionService,
                {
                    provide: AuthBaseService,
                    useClass: AuthService
                },
                AuthGuard
            ]
        };
    }
}
AuthModule.decorators = [
    { type: NgModule, args: [{
                imports: [],
                exports: [],
                entryComponents: [],
                providers: [
                    {
                        provide: HTTP_INTERCEPTORS,
                        useClass: AuthInterceptorHandlerService,
                        multi: true
                    }
                ],
                declarations: [AuthCallbackComponent]
            },] }
];

export { AuthCallbackComponent, AuthGuard, AuthModule, AuthService, ChangeLanguageComponent, FndModule, GlobalService, INTL_CONFIG, IntlLiteralsLoaderService, IntlModule, IntlService, LoadingComponent, LoadingModule, LoadingService, MessageComponent, MessageModule, MessageService, i18nFactory };
//# sourceMappingURL=totvs-log-web-foundation.js.map
