import { Router } from '@angular/router';
import { AuthBaseService, IAuthSettings, IAuthHandlerType } from 'totvs-log-base-foundation';
import { User } from 'oidc-client';
import { PoToolbarProfile } from '@portinari/portinari-ui';
export declare class AuthService extends AuthBaseService {
    private authSettings;
    private authType;
    /**
     * Método construtor da classe
     * @param authSettings Token de injeção com as configurações de autenticação
     * @param authType Token de injeção com o tipo de autenticação
     */
    constructor(authSettings: IAuthSettings, authType: IAuthHandlerType);
    /**
     * Adiciona nas uris de redirecionamento do processo oauth o host de origem, se um host não for informado
     * Isso é para permitir que o navegador identifique o domínio de redirecionamento
     * utilizando o caminho relativo
     *
     * @param authSettings as configurações de autenticação
     * @returns as configurações de autenticação com uris normalizadas para o processo de autenticação
     */
    static AddOriginUrls(authSettings: IAuthSettings): IAuthSettings;
    /**
     * Adiciona o host na url, se nenhum host estiver presente
     * @param host o host a ser adicionado
     * @param url url em que o host será adicionado
     * @returns a junção da url e host
     */
    static AddOriginUrl(host: string, url: string): string;
    /**
     * Verifica se está em atuação o fluxo de login e deveria ou não mostrar o menu
     */
    shouldShowMenu(router: Router): boolean;
    /**
     * Monta um PoToolbarProfile a partir do usuário. Sem suporte a avatar no momento
     */
    getToolbarProfile(user: User): PoToolbarProfile;
    /**
     * Método responsável executar o processo de login/autenticação
     */
    loginFlow(): Promise<void>;
    /**
     * Método responsável executar o processo de logout
     */
    logoutFlow(): Promise<void>;
}
