import { LoadingDefault, BaseMessage, I18nBaseService } from 'totvs-log-base-foundation';
import { ComponentFactoryResolver, ApplicationRef, Injector } from '@angular/core';
export declare class LoadingService extends LoadingDefault {
    private i18n;
    private componentFactoryResolver;
    private appRef;
    private injector;
    /**
     * Referência para o componente de loading no html
     */
    private loadingComponentRef;
    /**
     * Texto apresentado durante o loading
     */
    private loadingText;
    /**
     * Construtor do serviço
     * @param i18n Serviço de internacionalização
     * @param componentFactoryResolver Serviço que permite adicionar componentes dinamicamente
     * @param appRef Referência da aplicação Angular
     * @param injector Serviço de injeção de dependência
     */
    constructor(i18n: I18nBaseService, componentFactoryResolver: ComponentFactoryResolver, appRef: ApplicationRef, injector: Injector);
    /**
     * Exibe o loading, bloqueando a tela
     */
    showLoading(): void;
    /**
     * Esconde o loading, desbloqueando a tela
     */
    hideLoading(): void;
    /**
     * Atribui o texto que será apresentado
     * @param text Texto a ser apresentado, devendo ser um BaseMessage
     */
    setLoadingText(text: BaseMessage): void;
    /**
     * Método privado que cria dinamicamente o componente de loading no DOM
     */
    private createLoadingComponent;
}
