import { ComponentFactoryResolver, ApplicationRef, Injector } from '@angular/core';
import { ShowMessageOptions, AbstractShowMessage, I18nBaseService } from 'totvs-log-base-foundation';
import { PoNotificationService } from '@portinari/portinari-ui';
/**
 * Serviço de mensagens
 */
export declare class MessageService extends AbstractShowMessage {
    private poNotification;
    private componentFactoryResolver;
    private appRef;
    private injector;
    private i18n;
    /**
     * Atributo que recebe a instância da dialog criada
     */
    private messageDialogRef;
    /**
     * Método construtor da classe
     * @param poNotification Parâmetro do serviço de notificação do Po
     * @param componentFactoryResolver Parâmetro do serviço que permite adicionar componentes dinamicamente
     * @param appRef Parâmetro que recebe a referência da aplicação Angular.
     * @param injector Parâmetro do serviço de injeção de dependência
     * @param i18n  Parâmetro do serviço de internacionalização
     */
    constructor(poNotification: PoNotificationService, componentFactoryResolver: ComponentFactoryResolver, appRef: ApplicationRef, injector: Injector, i18n: I18nBaseService);
    /**
     * Método responsável por exibir uma mensagem, através de notificação ou dialog
     * @param showMessageOptions Parâmetro com as definições da mensagem
     */
    displayMessage(showMessageOptions: ShowMessageOptions): any;
    /**
     * Método responsável por exibir uma notificação em tela
     * @param showMessageOptions Parâmetro com as definições da mensagem
     */
    private showNotification;
    /**
     * Método por exibir uma dialog em tela
     * @param showMessageOptions Parâmetro com as definições da mensagem
     */
    private showDialog;
    /**
     * Método responsável por abrir no navegador uma nova tab com o link recebido por parâmetro
     * @param link Link de uma página externa
     */
    private openLink;
    /**
     * Método responsável por identificar se o link recebido é uma função ou página externa
     * @param showMessageOptions Parâmetro com as definições da mensagem
     */
    private processLinkAction;
    /**
     * Método responsável por criar o componente dialog
     */
    private createDialogComponent;
    /**
     * Método responsável por setar as opções do componente dialog
     * @param showMessageOptions Parâmetro com as definições da mensagem
     */
    private setDialogOptions;
}
